
#include <SPI.h>
#include <TFT_eSPI.h> // Hardware-specific library
#define TFT_GREY 0x5AEB
TFT_eSPI tft = TFT_eSPI();       // Invoke custom library

float sx = 0, sy = 1, mx = 1, my = 0, hx = -1, hy = 0;    // Saved H, M, S x & y multipliers
float sdeg=0, mdeg=0, hdeg=0;
uint16_t osx=120, osy=120, omx=120, omy=120, ohx=120, ohy=120;  // Saved H, M, S x & y coords
uint16_t x0=0, x1=0, yy0=0, yy1=0;
uint32_t targetTime = 0;                    // for next 1 second timeout
static uint8_t conv2d(const char* p); // Forward declaration needed for IDE 1.6.x

//uint8_t hh=conv2d(__TIME__), mm=conv2d(__TIME__+3), ss=conv2d(__TIME__+6);  // Get H, M, S from compile time
uint8_t hh=9, mm=25, ss=conv2d(__TIME__+6);  // Get H, M, S from compile time

bool initial = 1;

const int ledPin = 3;
const int freq = 5000;
const int ledChannel = 0;
const int resolution = 8;

void setup(void) {
 
  
  ledcSetup(ledChannel, freq, resolution);
  ledcAttachPin(ledPin, ledChannel);
  ledcWrite(ledChannel, 128);  // 0 to 255 = brightness
  //pinMode(3, OUTPUT);
  //digitalWrite(3, HIGH);
 
  tft.init();
  tft.setRotation(0);
  
  //tft.fillScreen(TFT_BLACK);
  //tft.fillScreen(TFT_RED);
  //tft.fillScreen(TFT_GREEN);
  //tft.fillScreen(TFT_BLUE);
  //tft.fillScreen(TFT_BLACK);
  //tft.fillCircle(120, 120, 10, TFT_BLUE);
  //tft.fillCircle(120, 121, 3, TFT_WHITE);

  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);  // Adding a background colour erases previous text automatically
  tft.fillCircle(120, 120, 10, TFT_GREY);
  

  // Draw 12 lines
  for(int i = 0; i<360; i+= 30) {
    sx = cos((i-90)*0.0174532925);
    sy = sin((i-90)*0.0174532925);
    x0 = sx*114+120;
    yy0 = sy*114+120;
    x1 = sx*100+120;
    yy1 = sy*100+120;

    tft.drawLine(x0, yy0, x1, yy1, TFT_WHITE);
    
  }
 
  tft.drawLine(220, 117, 239, 117, TFT_WHITE);
  tft.drawLine(220, 118, 239, 118, TFT_WHITE);
  tft.drawLine(220, 119, 239, 119, TFT_WHITE);
  tft.drawLine(220, 120, 239, 120, TFT_WHITE);
  tft.drawLine(220, 121, 239, 121, TFT_WHITE);
  tft.drawLine(220, 122, 239, 122, TFT_WHITE);
  tft.drawLine(220, 123, 239, 123, TFT_WHITE);

  tft.drawLine(0, 117, 19, 117, TFT_WHITE);
  tft.drawLine(0, 118, 19, 118, TFT_WHITE);
  tft.drawLine(0, 119, 19, 119, TFT_WHITE);
  tft.drawLine(0, 120, 19, 120, TFT_WHITE);
  tft.drawLine(0, 121, 19, 121, TFT_WHITE);
  tft.drawLine(0, 122, 19, 122, TFT_WHITE);
  tft.drawLine(0, 123, 19, 123, TFT_WHITE);

  tft.drawLine(117, 220, 117, 239, TFT_WHITE);
  tft.drawLine(118, 220, 118, 239, TFT_WHITE);
  tft.drawLine(119, 220, 119, 239, TFT_WHITE);
  tft.drawLine(120, 220, 120, 239, TFT_WHITE);
  tft.drawLine(121, 220, 121, 239, TFT_WHITE);
  tft.drawLine(122, 220, 122, 239, TFT_WHITE);
  tft.drawLine(123, 220, 123, 239, TFT_WHITE);



  tft.drawLine(111, 0, 111, 19, TFT_WHITE);
  tft.drawLine(112, 0, 112, 19, TFT_WHITE);
  tft.drawLine(113, 0, 113, 19, TFT_WHITE);
  tft.drawLine(114, 0, 114, 19, TFT_WHITE);
  tft.drawLine(115, 0, 115, 19, TFT_WHITE);
  tft.drawLine(116, 0, 116, 19, TFT_WHITE);
  tft.drawLine(117, 0, 117, 19, TFT_WHITE);

  tft.drawLine(120, 0, 120, 21, TFT_BLACK);

  tft.drawLine(123, 0, 123, 19, TFT_WHITE);
  tft.drawLine(124, 0, 124, 19, TFT_WHITE);
  tft.drawLine(125, 0, 125, 19, TFT_WHITE);
  tft.drawLine(126, 0, 126, 19, TFT_WHITE);
  tft.drawLine(127, 0, 127, 19, TFT_WHITE);
  tft.drawLine(128, 0, 128, 19, TFT_WHITE);
  tft.drawLine(129, 0, 129, 19, TFT_WHITE);



  // Draw 60 dots
  //for(int i = 0; i<360; i+= 6) {
  //  sx = cos((i-90)*0.0174532925);
  //  sy = sin((i-90)*0.0174532925);
  //  x0 = sx*12+120;
 //   yy0 = sy*12+120;
    // Draw minute markers
 //   tft.drawPixel(x0, yy0, TFT_WHITE);
    
    // Draw main quadrant dots
 //   if(i==0 || i==180) tft.fillCircle(x0, yy0, 2, TFT_WHITE);
 //   if(i==90 || i==270) tft.fillCircle(x0, yy0, 2, TFT_WHITE);
 // }



  // Draw text at position 120,260 using fonts 4
  // Only font numbers 2,4,6,7 are valid. Font 6 only contains characters [space] 0 1 2 3 4 5 6 7 8 9 : . - a p m
  // Font 7 is a 7 segment font and only contains characters [space] 0 1 2 3 4 5 6 7 8 9 : .
 
  tft.drawCentreString("VDO",120,70,4);






  targetTime = millis() + 1000; 
}

void loop() {



//uint16_t x, y;
//tft.getTouchRaw(&x, &y);
//tft.drawCentreString(x." ".y,120,160,4);




  if (targetTime < millis()) {
    targetTime += 1000;
    ss++;              // Advance second
    if (ss==60) {
      ss=0;
      mm++;            // Advance minute
      if(mm>59) {
        mm=0;
        hh++;          // Advance hour
        if (hh>23) {
          hh=0;
        }
      }
    }

    // Pre-compute hand degrees, x & y coords for a fast screen update
    sdeg = ss*6;                  // 0-59 -> 0-354
    mdeg = mm*6+sdeg*0.01666667;  // 0-59 -> 0-360 - includes seconds
    hdeg = hh*30+mdeg*0.0833333;  // 0-11 -> 0-360 - includes minutes and seconds
    hx = cos((hdeg-90)*0.0174532925);    
    hy = sin((hdeg-90)*0.0174532925);
    mx = cos((mdeg-90)*0.0174532925);    
    my = sin((mdeg-90)*0.0174532925);
    sx = cos((sdeg-90)*0.0174532925);    
    sy = sin((sdeg-90)*0.0174532925);

    if (ss==0 || initial) {
      initial = 0;
      // Erase hour and minute hand positions every minute
      tft.drawLine(ohx, ohy, 120, 121, TFT_BLACK);
      ohx = hx*62+121;    
      ohy = hy*62+121;
      tft.drawLine(omx, omy, 120, 121, TFT_BLACK);
      omx = mx*84+120;    
      omy = my*84+121;
    }

      // Redraw new hand positions, hour and minute hands not erased here to avoid flicker
      //tft.drawLine(osx, osy, 120, 121, TFT_BLACK);
      osx = sx*90+121;    
      osy = sy*90+121;
      //tft.drawLine(osx, osy, 120, 121, TFT_RED);
      tft.drawLine(ohx, ohy, 120, 121, TFT_WHITE);
      tft.drawLine(omx, omy, 120, 121, TFT_WHITE);
      //tft.drawLine(osx, osy, 120, 121, TFT_RED);

      //tft.drawCentreString("100 100 ",120,160,4);

      


    //tft.fillCircle(120, 121, 3, TFT_RED);
  }
}

static uint8_t conv2d(const char* p) {
  uint8_t v = 0;
  if ('0' <= *p && *p <= '9')
    v = *p - '0';
  return 10 * v + *++p - '0';
}

